@@ -14,6 +14,7 @@ from AccessControl import ClassSecurityInfo
 from AccessControl import getSecurityManager
 from BTrees.OOBTree import OOBTree
 from persistent.mapping import PersistentMapping
+from zope.i18n import translate
 
 from Products.Archetypes.atapi import *
 from Products.ATContentTypes.content.base import ATCTOrderedFolder
@@ -35,6 +36,7 @@ from Products.PloneSurvey.config import DEFAULT_SURVEY_INVITE
 
 from schemata import SurveySchema
 
+
 # Dumb class to work around bug in _getPropertyProviderForUser which
 # causes it to always operate on portal.acl_users
 class BasicPropertySheet:
@@ -94,7 +96,7 @@ class Survey(ATCTOrderedFolder):
         except TypeError:
             # BBB needed for Plone 3.3.5
             setupPlugins(self, out)
-        
+
         # Recreate mutable_properties but specify fields
         uf = self.acl_users
         pas = uf.manage_addProduct['PluggableAuthService']
@@ -131,7 +133,6 @@ class Survey(ATCTOrderedFolder):
                 'Survey Matrix',
                 'Survey Select Question',
                 'Survey Text Question',
-                'Survey Two Dimensional',
                 ]},
             full_objects=True)
         return questions
@@ -145,8 +146,7 @@ class Survey(ATCTOrderedFolder):
         results = portal_catalog.searchResults(portal_type = ['Survey Date Question',
                                                               'Survey Matrix Question',
                                                               'Survey Select Question',
-                                                              'Survey Text Question',
-                                                              'Survey Two Dimensional',],
+                                                              'Survey Text Question',],
                                                path = path,
                                                order = 'getObjPositionInParent')
         for result in results:
@@ -164,7 +164,6 @@ class Survey(ATCTOrderedFolder):
                 'Survey Matrix',
                 'Survey Select Question',
                 'Survey Text Question',
-                'Survey Two Dimensional',
                 ]},
             full_objects=True)
         for object in objects:
@@ -177,7 +176,6 @@ class Survey(ATCTOrderedFolder):
                         'Survey Date Question',
                         'Survey Select Question',
                         'Survey Text Question',
-                        'Survey Two Dimensional',
                         ]},
                     full_objects=True)
                 for sub_survey_object in sub_survey_objects:
@@ -188,20 +186,6 @@ class Survey(ATCTOrderedFolder):
                             full_objects=True)
                         for survey_matrix_object in survey_matrix_objects:
                             questions.append(survey_matrix_object)
-                    elif sub_survey_object.portal_type == 'Survey Two Dimensional':
-                        survey_2d_objects = sub_survey_object.getFolderContents(
-                            contentFilter={'portal_type' : 'Survey 2-Dimensional Question'},
-                            full_objects=True)
-                        for survey_2d_object in survey_2d_objects:
-                            questions.append(survey_2d_object)
-            elif object.portal_type == 'Survey Two Dimensional':
-                questions.append(object)
-                survey_2d_objects = object.getFolderContents(
-                    contentFilter={'portal_type' : 'Survey 2-Dimensional Question'},
-                    full_objects=True)
-                for survey_2d_object in survey_2d_objects:
-                    questions.append(survey_2d_object)
-                # XXX should check if comment is present
             elif object.portal_type == 'Survey Matrix':
                 questions.append(object)
                 survey_matrix_objects = object.getFolderContents(
@@ -252,6 +236,14 @@ class Survey(ATCTOrderedFolder):
                 questions.append(object)
         return questions
 
+    security.declareProtected(permissions.View, 'hasDateQuestion')
+    def hasDateQuestion(self):
+        """Return true if there is a date question in this part of the survey to import the js"""
+        objects = self.getFolderContents(contentFilter={'portal_type':'Survey Date Question'})
+        if objects:
+            return True
+        return False
+
     security.declareProtected(permissions.View, 'getNextPage')
     def getNextPage(self):
         """Return the next page of the survey"""
@@ -298,9 +290,9 @@ class Survey(ATCTOrderedFolder):
         if user is not None:
             portal_registration = getToolByName(self, 'portal_registration')
             pw = portal_registration.generatePassword()
-            
+
             self.acl_users.userFolderEditUser(userid, pw, user.getRoles(), user.getDomains(), key=pw)
-            
+
             # Set key
             props = acl_users.mutable_properties.getPropertiesForUser(user)
             props = BasicPropertySheet(props)
@@ -429,6 +421,8 @@ class Survey(ATCTOrderedFolder):
             respondents = self.respondents
         except AttributeError:
             self.reset()
+        if self.respondents.has_key(user_id):
+            return
         self.respondents[user_id] = PersistentMapping(start=DateTime(),
                                                       ip_address=self.getRemoteIp(),
                                                       end='')
@@ -455,7 +449,7 @@ class Survey(ATCTOrderedFolder):
             return full_name
         return member.id
 
-    security.declareProtected(permissions.ViewSurveyResults, 'getRespondents')
+    security.declareProtected(permissions.ViewSurveyResults, 'getAnswersByUser')
     def getAnswersByUser(self, userid):
         """Return a set of answers by user id"""
         questions = self.getAllQuestionsInOrder()
@@ -487,11 +481,15 @@ class Survey(ATCTOrderedFolder):
         mtool = getToolByName(self, 'portal_membership')
         member = mtool.getAuthenticatedMember()
         user_id = member.getMemberId()
-        return self.resetForUser(user_id)
+        return self._resetForUser(user_id)
 
     security.declareProtected(permissions.ModifyPortalContent,
                               'resetForUser')
     def resetForUser(self, userid):
+        """Remove answer for a single user"""
+        self._resetForUser(userid)
+
+    def _resetForUser(self, userid):
         """Remove answer for a single user"""
         completed = self.getCompletedFor()
         if userid in completed:
@@ -513,14 +511,20 @@ class Survey(ATCTOrderedFolder):
         properties = self.portal_properties.site_properties
         mTo = self.getSurveyNotificationEmail()
         mFrom = properties.email_from_address
-        mSubj = '[%s] New survey submitted' % self.Title()
+        mSubj = translate(_('[${survey_title}] New survey submitted',
+                            mapping={'survey_title': self.Title()}),
+                context=self.REQUEST)
         message = []
-        message.append('Survey %s.' % self.Title())
-        message.append('has been completed by user: %s.' % userid)
-        message.append(self.absolute_url() + '/survey_view_results')
+        message.append(translate(_('Survey ${survey_title}',
+                                 mapping={'survey_title': self.Title()}),
+                        context=self.REQUEST))
+        message.append(translate(_('has been completed by user: ${userid}',
+                                 mapping={'userid': userid}),
+                       context=self.REQUEST))
+        message.append(self.absolute_url() + '/@@Products.PloneSurvey.survey_view_results')
         mMsg = '\n\n'.join(message)
         try:
-            self.MailHost.send(mMsg, mTo, mFrom, mSubj)
+            self.MailHost.send(mMsg.encode('utf-8'), mTo, mFrom, mSubj)
         except ConflictError:
             raise
         except:
@@ -628,7 +632,7 @@ class Survey(ATCTOrderedFolder):
             email_from_address = self.email_from_address
         email_body = self.getEmailInvite()
         email_body = email_body.replace('**Name**', user_details['fullname'])
-        survey_url = self.absolute_url() + '/login_form_bridge?email=' + email_address + '&amp;key=' + user_details['key']
+        survey_url = self.absolute_url() + '/login_form_bridge?email=' + email_address + '&amp;key=' + urllib.quote(user_details['key'])
         email_body = email_body.replace('**Survey**', '<a href="' + survey_url + '">' + self.Title() + '</a>')
         mail_text = self.survey_send_invite_template(
             user=user,
@@ -674,24 +678,49 @@ class Survey(ATCTOrderedFolder):
             self.createLocalPas()
         return self.acl_users
 
-    security.declareProtected(permissions.View, 'buildSpreadsheetUrl')
-    def buildSpreadsheetUrl(self):
+    security.declareProtected(permissions.ViewSurveyResults, 'setCsvHeaders')
+    def setCsvHeaders(self, filetype='csv'):
+        """Set the CSV headers"""
+        REQUEST = self.REQUEST
+        file = self.buildSpreadsheetUrl(filetype=filetype)
+        REQUEST.RESPONSE.setHeader('Content-Type','text/x-comma-separated-values; charset=utf-8')
+        REQUEST.RESPONSE.setHeader('Content-disposition','attachment; filename=%s' % file)
+        return REQUEST
+
+    security.declareProtected(permissions.ViewSurveyResults, 'buildSpreadsheetUrl')
+    def buildSpreadsheetUrl(self, filetype='csv'):
         """Create a filename for the spreadsheets"""
         date = DateTime().strftime("%Y-%m-%d")
         id = self.getId()
         id = "%s-%s" % (id, date)
-        url = "%s.csv" % id
+        url = "%s.%s" % (id, filetype)
         return url
 
+    security.declareProtected(permissions.ViewSurveyResults, 'spreadsheet2')
+    def spreadsheet2(self):
+        """Return spreadsheet 2"""
+        self.setCsvHeaders()
+        return self.buildSpreadsheet2()
+
+    security.declareProtected(permissions.ViewSurveyResults, 'spreadsheet2_tab')
+    def spreadsheet2_tab(self):
+        """Return spreadsheet 2 tab"""
+        self.setCsvHeaders(filetype='tsv')
+        dialect = csv.excel_tab
+        return self.buildSpreadsheet2(dialect)
+
     security.declareProtected(permissions.ViewSurveyResults, 'buildSpreadsheet2')
-    def buildSpreadsheet2(self):
-        """Build spreadsheet 2."""
+    def buildSpreadsheet2(self, dialect=csv.excel):
+        """Build spreadsheet 2.
+            excel_tab
+            excel
+        """
         data = StringIO()
-        sheet = csv.writer(data)
+        sheet = csv.writer(data, dialect=dialect, quoting=csv.QUOTE_ALL)
         questions = self.getAllQuestionsInOrder()
-        
+
         sheet.writerow(('user',) + tuple(q.Title() for q in questions) + ('completed',))
-        
+
         for user in self.getRespondents():
             if self.getConfidential():
                 row = ['Anonymous']
@@ -704,20 +733,46 @@ class Survey(ATCTOrderedFolder):
                     if not (isinstance(answer, str) or isinstance(answer, int)):
                         # It's a sequence, filter out empty values
                         answer = ', '.join(filter(None, answer))
-                row.append(answer)
-            
+                row.append(answer.replace('"',"'").replace('\r\n', ' '))
+
             row.append(self.checkCompletedFor(user) and 'Completed' or 'Not Completed')
-            
+
             sheet.writerow(row)
-        
+
         return data.getvalue()
 
+    security.declareProtected(permissions.ViewSurveyResults, 'spreadsheet3')
+    def spreadsheet3(self):
+        """Return spreadsheet 3"""
+        self.setCsvHeaders()
+        return self.buildSpreadsheet3()
+
+    security.declareProtected(permissions.ViewSurveyResults, 'get_all_questions_in_order_filtered')
+    def get_all_questions_in_order_filtered(self, include_sub_survey=False, ignore_meta_types=[], ignore_input_types=[], restrict_meta_types=[]):
+        """This is only used in buildSpreadsheet3, and should be moved into a nother method."""
+        questions = self.getAllQuestionsInOrder(include_sub_survey=include_sub_survey)
+        result = []
+        for question in questions:
+            ok = True
+            if ignore_meta_types:
+                if question.meta_type in ignore_meta_types:
+                    ok = ok and False
+            if ignore_input_types:
+                if question.getInputType() in ignore_input_types:
+                    ok = ok and False
+            if restrict_meta_types:
+                if question.meta_type not in restrict_meta_types:
+                    ok = ok and False
+            if ok:
+                result.append(question)
+        return result
+
     security.declareProtected(permissions.ViewSurveyResults, 'buildSpreadsheet3')
     def buildSpreadsheet3(self):
         """Build spreadsheet 3."""
         data = StringIO()
         sheet = csv.writer(data)
-        questions = self.get_all_questions_in_order_filtered(ignore_meta_types=['SurveyTwoDimensional','SurveyMatrix'])
+        questions = self.get_all_questions_in_order_filtered(ignore_meta_types=['SurveyMatrix',])
         sheet.writerow(('user',) + tuple(q.Title() for q in questions) + ('completed',))
         for user in self.getRespondents():
             if self.getConfidential():
@@ -735,6 +790,8 @@ class Survey(ATCTOrderedFolder):
                     options = question.getQuestionOptions()
                     answerList = question.getAnswerFor(user)
                     if answerList and not isinstance(answerList, str):
+                        if not isinstance(answerList, list):
+                            answerList = [answerList]
                         for option in options:
                             if answerList.count(option) > 0:
                                 answer += '1;'
@@ -746,7 +803,10 @@ class Survey(ATCTOrderedFolder):
                     else:
                         answer = ''
                 else:
-                    options = question.getQuestionOptions()
+                    if not hasattr(question, 'getQuestionOptions'):
+                        options = question.title
+                    else:
+                        options = question.getQuestionOptions()
                     answerLabel = question.getAnswerFor(user)
                     answer = str(len(options))
                     i = 0
@@ -762,6 +822,12 @@ class Survey(ATCTOrderedFolder):
             sheet.writerow(row)
         return data.getvalue()
 
+    security.declareProtected(permissions.ViewSurveyResults, 'summary_spreadsheet')
+    def summary_spreadsheet(self):
+        """Return summary spreadsheet"""
+        self.setCsvHeaders()
+        return self.buildSummarySpreadsheet()
+
     security.declareProtected(permissions.ModifyPortalContent, 'buildSummarySpreadsheet')
     def buildSummarySpreadsheet(self):
         """Build the summary spreadsheet."""
@@ -785,6 +851,16 @@ class Survey(ATCTOrderedFolder):
                     sheet.writerow(row)
         return data.getvalue()
 
+    security.declareProtected(permissions.ViewSurveyResults, 'spreadsheet_select')
+    def spreadsheet_select(self):
+        """Return spreadsheet select"""
+        self.setCsvHeaders()
+        try:
+            answers = self.REQUEST.form['answers']
+        except KeyError:
+            return self.buildSelectSpreadsheet()
+        return self.buildSelectSpreadsheet(boolean=True)
+
     security.declareProtected(permissions.ViewSurveyResults, 'buildSelectSpreadsheet')
     def buildSelectSpreadsheet(self, boolean=False):
         """Build the select spreadsheet."""
@@ -818,6 +894,8 @@ class Survey(ATCTOrderedFolder):
                         else:
                             row.append('')
                     else:
+                        if type(answer)==int:
+                            answer=str(answer)
                         if options[i] in answer:
                             if boolean:
                                 row.append(1)
@@ -832,19 +910,8 @@ class Survey(ATCTOrderedFolder):
             sheet.writerow(row)
         return data.getvalue()
 
-    security.declareProtected(permissions.ManagePortal, 'fixSurveyResults')
-    def fixSurveyResults(self):
-        """Fix survey results"""
-        return # XXX this method does not work yet
-        questions = self.getAllQuestions()
-        for question in questions:
-            if question in ['Survey Matrix Question', 'Survey Select Question']:
-                should_be_integer = question.getLikertOptions() and True or False
-                raise str(should_be_integer)
-                answers = question.answers
-        return 'done'
-
-    security.declareProtected(permissions.ManagePortal, 'openFile')
+    # TODO next two methods are still needed for the tests, but should be removed and the tests fixed
+    security.declareProtected(permissions.ModifyPortalContent, 'openFile')
     def openFile(self):
         """open the file, and return the file contents"""
         data_path = os.path.abspath('import')
@@ -861,7 +928,7 @@ class Survey(ATCTOrderedFolder):
         data_catch.close()
         return input
 
-    security.declareProtected(permissions.ManagePortal, 'uploadRespondents')
+    security.declareProtected(permissions.ModifyPortalContent, 'uploadRespondents')
     def uploadRespondents(self, input=None):
         """upload the respondents"""
         if input is None:
@@ -876,6 +943,7 @@ class Survey(ATCTOrderedFolder):
                 errors.append(user)
         return errors
 
+    security.declareProtected(permissions.ModifyPortalContent, 'pre_validate')
     def pre_validate(self, REQUEST, errors):
         """ checks captcha """
         product_installed = self.portal_quickinstaller.isProductInstalled('quintagroup.plonecaptchas')
